@@ -21,6 +21,7 @@ import os
 import signal
 import subprocess
 import functools
+import itertools
 from functools import partial
 
 global NUM_CPUS
@@ -42,6 +43,8 @@ TARGET_DIR = None
 ALLOWED_EXTENSIONS = None
 NEXT_SCRIPT = None
 
+global action
+
 def _quote(s):
     return ''.join(["'",s,"'"])
 
@@ -378,10 +381,8 @@ def main_body(action, filename_parser=None, argv=None):
 
         # Check num-cpus
         if options.has_key('p'):
-            global NUM_CPUS
             NUM_CPUS = int(options['p'])
         elif options.has_key('num-cpus'):
-            global NUM_CPUS
             NUM_CPUS = int(options['num-cpus'])
 
         fp_kwargs['source_dir'] = SOURCE_DIR
@@ -402,9 +403,7 @@ def main_body(action, filename_parser=None, argv=None):
 
         # If a FilenameParser is not provided, use the built-in
         # and allow script to specify TARGET_DIR
-        if filename_parser is None:
-            filename_parser = FilenameParser
-        filename_parser = partial(filename_parser, **fp_kwargs)
+        if filename_parser is None: filename_parser = FilenameParser
 
         # Check if we need to find the files or if any were specified
         # Also check if we have a proper filename parser
@@ -420,7 +419,8 @@ def main_body(action, filename_parser=None, argv=None):
                 if ALLOWED_EXTENSIONS is not None:
                     print_debug('Valid file extensions are',
                                 ' '.join(ALLOWED_EXTENSIONS))
-            sequence = find_files(filename_parser, **verbose_kwargs)
+            sequence = find_files(partial(filename_parser, **fp_kwargs),
+                                  **verbose_kwargs)
         else:
             sequence = []
         
@@ -431,7 +431,7 @@ def main_body(action, filename_parser=None, argv=None):
                             ' '.join(ALLOWED_EXTENSIONS))
 
         # Also includes explicitly mentioned files
-        sequence += [filename_parser(pirate) for pirate in args]
+        sequence += [filename_parser(pirate, **fp_kwargs) for pirate in args]
         filtered_sequence = [x for x in sequence if not x.is_invalid]
 
     except Usage, err:
@@ -460,7 +460,12 @@ def main_body(action, filename_parser=None, argv=None):
 def debug_action(action):
     return lambda item: action(item, debug=True)
 
+
+
 def spawn_workers(action, sequence, **kwargs):
+    def uncurry_action(item):
+        return action(item[1], **item[0])
+
     verbose = kwargs['verbose']
     debug = kwargs['debug']
     silent = kwargs['silent']
@@ -479,7 +484,8 @@ def spawn_workers(action, sequence, **kwargs):
                         'debug output may get mangeled')
         signal.signal(signal.SIGCHLD, signal.SIG_DFL)
         p = multiprocessing.Pool(processes=NUM_CPUS)
-        stdouts = p.map(partial(action, **kwargs), sequence)
-        stdouts = [x for x in stdouts if x is not None]
+        results = [p.apply_async(action, (item,), kwargs) for item in sequence]
+        stdouts = (result.get() for result in results)
+        stdouts_good = itertools.ifilter(lambda x: x is not None, stdouts)
         if not silent and stdouts is not None:
-            print >>sys.stdout, os.linesep.join(stdouts)
+            print >>sys.stdout, os.linesep.join(stdouts_good)
